@@ -839,7 +839,7 @@ macro_rules! float_method_msvc {
         return unsafe { core::intrinsics::$intr($f $(,$i)*) };
 
         #[cfg(all(not(feature = "std"), target_env = "msvc"))]
-        return ($f as $tl).$meth() as $ts;
+        return $tl::$meth($f as $tl $(,$i as $tl)*) as $ts;
     })
 }
 
@@ -926,7 +926,7 @@ impl Float for f32 {
 
     #[inline]
     fn powf(self, n: f32) -> f32 {
-        float_method_msvc!(self, f32, f64, powf, powf32, n as f32)
+        float_method_msvc!(self, f32, f64, powf, powf32, n)
     }
 
     #[inline]
