@@ -2,8 +2,8 @@
 
 #![allow(dead_code)]
 
-use lib::{cmp, iter, marker, ops, ptr, slice};
-use stackvector;
+use lib::{cmp, iter, marker, mem, ops, ptr, slice};
+use arrayvec;
 use super::bound::Bound;
 use super::pointer_methods::PointerMethods;
 use super::range_bounds::RangeBounds;
@@ -12,6 +12,142 @@ use super::slice_index::SliceIndex;
 #[cfg(all(feature = "correct", feature = "radix"))]
 use lib::Vec;
 
+// ARRVEC
+
+/// Macro to automate simplify the creation of an ArrayVec.
+#[macro_export]
+macro_rules! arrvec {
+    // This only works if the ArrayVec is the same size as the input array.
+    ($elem:expr; $n:expr) => ({
+        $crate::arrayvec::ArrayVec::from([$elem; $n])
+    });
+    // This just repeatedly calls `push`. I don't believe there's a concise way to count the number of expressions.
+    ($($x:expr),*$(,)*) => ({
+        // Allow an unused mut variable, since if the sequence is empty,
+        // the vec will never be mutated.
+        #[allow(unused_mut)] {
+            let mut vec = $crate::arrayvec::ArrayVec::new();
+            $(vec.push($x);)*
+            vec
+        }
+    });
+}
+
+// INSERT MANY
+
+/// Insert multiple elements at position `index`.
+///
+/// Shifts all elements before index to the back of the iterator.
+/// It uses size hints to try to minimize the number of moves,
+/// however, it does not rely on them. We cannot internally allocate, so
+/// if we overstep the lower_size_bound, we have to do extensive
+/// moves to shift each item back incrementally.
+///
+/// This implementation is adapted from [`smallvec`], which has a battle-tested
+/// implementation that has been revised for at least a security advisory
+/// warning. Smallvec is similarly licensed under an MIT/Apache dual license.
+///
+/// [`smallvec`]: https://github.com/servo/rust-smallvec
+pub fn insert_many<V, T, I>(vec: &mut V, index: usize, iterable: I)
+    where V: VecLike<T>,
+          I: iter::IntoIterator<Item=T>
+{
+    let mut iter = iterable.into_iter();
+    if index == vec.len() {
+        return vec.extend(iter);
+    }
+
+    let (lower_size_bound, _) = iter.size_hint();
+    assert!(lower_size_bound <= core::isize::MAX as usize); // Ensure offset is indexable
+    assert!(index + lower_size_bound >= index); // Protect against overflow
+
+    let mut num_added = 0;
+    let old_len = vec.len();
+    assert!(index <= old_len);
+
+    unsafe {
+        // Reserve space for `lower_size_bound` elements.
+        vec.reserve(lower_size_bound);
+        let start = vec.as_mut_ptr();
+        let ptr = start.add(index);
+
+        // Move the trailing elements.
+        ptr::copy(ptr, ptr.add(lower_size_bound), old_len - index);
+
+        // In case the iterator panics, don't double-drop the items we just copied above.
+        vec.set_len(0);
+        let mut guard = DropOnPanic {
+            start,
+            skip: index..(index + lower_size_bound),
+            len: old_len + lower_size_bound,
+        };
+
+        while num_added < lower_size_bound {
+            let element = match iter.next() {
+                Some(x) => x,
+                None => break,
+            };
+            let cur = ptr.add(num_added);
+            ptr::write(cur, element);
+            guard.skip.start += 1;
+            num_added += 1;
+        }
+
+        if num_added < lower_size_bound {
+            // Iterator provided fewer elements than the hint. Move the tail backward.
+            ptr::copy(
+                ptr.add(lower_size_bound),
+                ptr.add(num_added),
+                old_len - index,
+            );
+        }
+        // There are no more duplicate or uninitialized slots, so the guard is not needed.
+        vec.set_len(old_len + num_added);
+        mem::forget(guard);
+    }
+
+    // Insert any remaining elements one-by-one.
+    for element in iter {
+        vec.insert(index + num_added, element);
+        num_added += 1;
+    }
+
+    struct DropOnPanic<T> {
+        start: *mut T,
+        skip: ops::Range<usize>, // Space we copied-out-of, but haven't written-to yet.
+        len: usize,
+    }
+
+    impl<T> DropOnPanic<T> {
+        // Contains methods for Rustc versions < 1.35.0.
+        // Remove when support is dropped for < 1.35.0.
+        #[inline]
+        fn contains(&self, item: &usize) -> bool {
+            (match self.skip.start_bound() {
+                ops::Bound::Included(ref start) => *start <= item,
+                ops::Bound::Excluded(ref start) => *start < item,
+                ops::Bound::Unbounded => true,
+            }) && (match self.skip.end_bound() {
+                ops::Bound::Included(ref end) => item <= *end,
+                ops::Bound::Excluded(ref end) => item < *end,
+                ops::Bound::Unbounded => true,
+            })
+        }
+    }
+
+    impl<T> Drop for DropOnPanic<T> {
+        fn drop(&mut self) {
+            for i in 0..self.len {
+                if !self.contains(&i) {
+                    unsafe {
+                        ptr::drop_in_place(self.start.add(i));
+                    }
+                }
+            }
+        }
+    }
+}
+
 // REMOVE_MANY
 
 /// Remove many elements from a vec-like container.
@@ -217,17 +353,17 @@ impl<T> SliceLikeImpl<T> for Vec<T> {
     }
 }
 
-impl<A: stackvector::Array> SliceLikeImpl<A::Item> for stackvector::StackVec<A> {
+impl<A: arrayvec::Array> SliceLikeImpl<A::Item> for arrayvec::ArrayVec<A> {
     // AS SLICE
 
     #[inline]
     fn as_slice(&self) -> &[A::Item] {
-        stackvector::StackVec::as_slice(self)
+        arrayvec::ArrayVec::as_slice(self)
     }
 
     #[inline]
     fn as_mut_slice(&mut self) -> &mut [A::Item] {
-        stackvector::StackVec::as_mut_slice(self)
+        arrayvec::ArrayVec::as_mut_slice(self)
     }
 }
 
@@ -998,7 +1134,7 @@ impl<T> SliceLike<T> for Vec<T> {
     }
 }
 
-impl<A: stackvector::Array> SliceLike<A::Item> for stackvector::StackVec<A> {
+impl<A: arrayvec::Array> SliceLike<A::Item> for arrayvec::ArrayVec<A> {
     // GET
 
     /// Get an immutable reference to item at index.
@@ -1250,32 +1386,32 @@ impl<T> VecLike<T> for Vec<T> {
     }
 }
 
-impl<A: stackvector::Array> VecLike<A::Item> for stackvector::StackVec<A> {
+impl<A: arrayvec::Array> VecLike<A::Item> for arrayvec::ArrayVec<A> {
     #[inline]
-    fn new() -> stackvector::StackVec<A> {
-        stackvector::StackVec::new()
+    fn new() -> arrayvec::ArrayVec<A> {
+        arrayvec::ArrayVec::new()
     }
 
     #[inline]
-    fn with_capacity(capacity: usize) -> stackvector::StackVec<A> {
-        let mut v = stackvector::StackVec::new();
+    fn with_capacity(capacity: usize) -> arrayvec::ArrayVec<A> {
+        let mut v = arrayvec::ArrayVec::new();
         v.reserve(capacity);
         v
     }
 
     #[inline]
     fn capacity(&self) -> usize {
-        stackvector::StackVec::capacity(self)
+        arrayvec::ArrayVec::capacity(self)
     }
 
     #[inline]
     fn reserve(&mut self, capacity: usize) {
-        assert!(capacity < self.capacity());
+        assert!(self.len() + capacity <= self.capacity());
     }
 
     #[inline]
     fn reserve_exact(&mut self, capacity: usize) {
-        assert!(capacity < self.capacity());
+        assert!(self.len() + capacity <= self.capacity());
     }
 
     #[inline]
@@ -1284,47 +1420,47 @@ impl<A: stackvector::Array> VecLike<A::Item> for stackvector::StackVec<A> {
 
     #[inline]
     fn truncate(&mut self, len: usize) {
-        stackvector::StackVec::truncate(self, len)
+        arrayvec::ArrayVec::truncate(self, len)
     }
 
     #[inline]
     unsafe fn set_len(&mut self, new_len: usize) {
-        stackvector::StackVec::set_len(self, new_len);
+        arrayvec::ArrayVec::set_len(self, new_len);
     }
 
     #[inline]
     fn swap_remove(&mut self, index: usize) -> A::Item {
-        stackvector::StackVec::swap_remove(self, index)
+        arrayvec::ArrayVec::swap_remove(self, index)
     }
 
     #[inline]
     fn insert(&mut self, index: usize, element: A::Item) {
-        stackvector::StackVec::insert(self, index, element)
+        arrayvec::ArrayVec::insert(self, index, element)
     }
 
     #[inline]
     fn remove(&mut self, index: usize) -> A::Item {
-        stackvector::StackVec::remove(self, index)
+        arrayvec::ArrayVec::remove(self, index)
     }
 
     #[inline]
     fn push(&mut self, value: A::Item) {
-        stackvector::StackVec::push(self, value);
+        arrayvec::ArrayVec::push(self, value);
     }
 
     #[inline]
     fn pop(&mut self) -> Option<A::Item> {
-        stackvector::StackVec::pop(self)
+        arrayvec::ArrayVec::pop(self)
     }
 
     #[inline]
     fn clear(&mut self) {
-        stackvector::StackVec::clear(self);
+        arrayvec::ArrayVec::clear(self);
     }
 
     #[inline]
     fn insert_many<I: iter::IntoIterator<Item=A::Item>>(&mut self, index: usize, iterable: I) {
-        stackvector::StackVec::insert_many(self, index, iterable)
+        insert_many(self, index, iterable)
     }
 
     #[inline]
@@ -1362,17 +1498,25 @@ impl<T> CloneableVecLike<T> for Vec<T>
     }
 }
 
-impl<A: stackvector::Array> CloneableVecLike<A::Item> for stackvector::StackVec<A>
-    where A::Item: Clone + Copy + Send
+impl<A: arrayvec::Array> CloneableVecLike<A::Item> for arrayvec::ArrayVec<A>
+    where A: Send,
+          A::Index: Send,
+          A::Item: Clone + Copy + Send
 {
     #[inline]
     fn extend_from_slice(&mut self, other: &[A::Item]) {
-        stackvector::StackVec::extend_from_slice(self, other)
+        self.extend(other.iter().cloned())
     }
 
     #[inline]
     fn resize(&mut self, len: usize, value: A::Item) {
-        stackvector::StackVec::resize(self, len, value)
+        assert!(len <= self.capacity());
+        let old_len = self.len();
+        if len > old_len {
+            self.extend(iter::repeat(value).take(len - old_len));
+        } else {
+            self.truncate(len);
+        }
     }
 }
 
@@ -1383,6 +1527,18 @@ impl<A: stackvector::Array> CloneableVecLike<A::Item> for stackvector::StackVec<
 mod tests {
     use super::*;
 
+    #[test]
+    fn test_insert_many() {
+        type V = arrayvec::ArrayVec<[u8; 8]>;
+        let mut v: V = V::new();
+        for x in 0..4 {
+            v.push(x);
+        }
+        assert_eq!(v.len(), 4);
+        v.insert_many(1, [5, 6].iter().cloned());
+        assert_eq!(&v[..], &[0, 5, 6, 1, 2, 3]);
+    }
+
     #[cfg(all(feature = "correct", feature = "radix"))]
     #[test]
     fn remove_many_test() {
